Enum Verdict Copy item path Source pub enum Verdict {
Proved,
RefutedBy(Constraint ),
RefutedAt(Point ),
Unknown,
}Expand description Solver response.
t0 refutations provide a trivial !goal given non-negative variables;
t1 refutations provide a counterexample point;
t2 is best-effort and cannot refute.
Performs copy-assignment from
source.
Read more Formats the value using the given formatter.
Read more Tests for self and other values to be equal, and is used by ==.
Tests for !=. The default implementation is almost always sufficient,
and should not be overridden without very good reason.
Immutably borrows from an owned value.
Read more Mutably borrows from an owned value.
Read more 🔬 This is a nightly-only experimental API. (clone_to_uninit)
Performs copy-assignment from
self to
dest.
Read more Returns the argument unchanged.
Calls U::from(self).
That is, this conversion is whatever the implementation of
From <T> for U chooses to do.
The resulting type after obtaining ownership.
Creates owned data from borrowed data, usually by cloning.
Read more Uses borrowed data to replace owned data, usually by cloning.
Read more The type returned in the event of a conversion error.
Performs the conversion.
The type returned in the event of a conversion error.
Performs the conversion.
Perform a zero-cost type-safe upcast to a wider ref type within the Wasm
bindgen generics type system.
Read more Perform a zero-cost type-safe upcast to a wider type within the Wasm
bindgen generics type system.
Read more