pub struct Z3 {
pub queries: usize,
pub z3_calls: usize,
/* private fields */
}Expand description
A cached Z3 pipe, using either the command line or a JavaScript callback.
Fields
queries: usizeObligations submitted.
z3_calls: usizeObligations passed to z3.
Implementations
Sourceimpl Z3
impl Z3
Sourcepub fn new_cli(cache_dir: Option<PathBuf>) -> Option<Self>
pub fn new_cli(cache_dir: Option<PathBuf>) -> Option<Self>
Expect the z3 command to be available in the environment.
Optionally provide a filesystem cache for model output.
Sourcepub fn new_wasm(callback: Function) -> Self
pub fn new_wasm(callback: Function) -> Self
Pass a javascript callback string->string for smt2 input.
Sourcepub fn entails_lia(
&mut self,
facts: &[Constraint],
goal: &Constraint,
nonneg: &[Var],
names: &[&str],
) -> Verdict
pub fn entails_lia( &mut self, facts: &[Constraint], goal: &Constraint, nonneg: &[Var], names: &[&str], ) -> Verdict
Try to prove a goal given facts by linearizing the whole system.